using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Highlighting(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
        {
            StandAloneWindowKey myWindowKey = (StandAloneWindowKey)wk;
            /* Sample: Highlight styles
            * This sample will show you how to:
            * - Load an HSF file
            * - Load an image and use it to create a texture
            * - Create an opaque style
            * - Create a transparent style
            * - Create a textured style
            *
            * Read more about the topics covered in this sample in our Programming Guide
            * - section 6.4:	Highlighting
            * - section 4.3:    Defining and using styles
            */

            //Setup four sub windows
            SegmentKey one = modelKey.Subsegment();
            one.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(-1, 0, 0, 1));
            one.InsertDistantLight(new Vector(0.5f, 0.5f, 0));
            SegmentKey two = modelKey.Subsegment();
            two.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(0, 1, 0, 1));
            two.InsertDistantLight(new Vector(0.5f, 0.5f, 0));
            SegmentKey three = modelKey.Subsegment();
            three.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(-1, 0, -1, 0));
            three.InsertDistantLight(new Vector(0.5f, 0.5f, 0));
            SegmentKey four = modelKey.Subsegment();
            four.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(0, 1, -1, 0));
            four.InsertDistantLight(new Vector(0.5f, 0.5f, 0));

            HPS.ImageKit imageKit = new HPS.ImageKit();
            SegmentKey root = Database.CreateRootSegment();

            try
            {
                /* This next block of code shows you how to load an HSF file
                * 1. Create an ImportOptionsKit and set the segment in which you want the file to be loaded
                * 2. Use the Stream::File::Import method to load the desired file with the options you specified
                * 3. Since this operation happens on a separate thread, wait for it to complete before proceeding
                * 4. Use a try-catch block to capture any Stream IO exceptions */
                HPS.Stream.ImportOptionsKit importOptionsKit = new HPS.Stream.ImportOptionsKit();
                importOptionsKit.SetSegment(root);
                String filename = INPUT_FOLDER + "/bnc.hsf";

                HPS.Stream.ImportNotifier notifier = HPS.Stream.File.Import(filename, importOptionsKit);
                notifier.Wait();

                /* This block of code shows you how to load an iamge from a file
                * 1. Create an ImportOptionsKit and set the image format
                * 2. Use the Image::File::Import method to load the desired file with the options you specified
                * 3. Use a try-catch block to capture any IO exceptions */
                filename = INPUT_FOLDER + "/pattern.jpg";
                HPS.Image.ImportOptionsKit iok = new HPS.Image.ImportOptionsKit();
                iok.SetFormat(HPS.Image.Format.Jpeg);
                imageKit = HPS.Image.File.Import(filename, iok);
            }
            catch (HPS.IOException)
            {
                // handle IO errors here
            }

            //Include the loaded model into the four sub windows
            one.IncludeSegment(root);
            two.IncludeSegment(root);
            three.IncludeSegment(root);
            four.IncludeSegment(root);

            // create a portfolio to store styles and the texture
            HPS.PortfolioKey myPortfolio = HPS.Database.CreatePortfolio();

            // create an ImageDefinition from the image loaded earlier. an image definition
            // is required to create a TextureDefinition
            HPS.ImageDefinition imageDefinition = myPortfolio.DefineImage("my_image", imageKit);
            HPS.TextureOptionsKit textureOptionsKit = new HPS.TextureOptionsKit();

            // see the table below about parameterization sources
            textureOptionsKit.SetParameterizationSource(HPS.Material.Texture.Parameterization.UV);

            // makes the image into a texture
            myPortfolio.DefineTexture("my_texture", imageDefinition, textureOptionsKit);

            modelKey.GetModellingMatrixControl().Rotate(-90, 30, 0).Scale(0.6f, 0.6f, 0.6f);


            // create the opaque red style
            HPS.SegmentKey redStyleSegment = HPS.Database.CreateRootSegment();
            HPS.NamedStyleDefinition red_style = myPortfolio.DefineNamedStyle("red_style", redStyleSegment);
            redStyleSegment.GetMaterialMappingControl().SetFaceColor(new HPS.RGBAColor(1.0f, 0, 0));

            // create the transparent style. it is a 1/2 transparent red style
            HPS.SegmentKey transparentStyleSegment = HPS.Database.CreateRootSegment();
            HPS.NamedStyleDefinition transparent_style = myPortfolio.DefineNamedStyle("transparent_style", transparentStyleSegment);
            transparentStyleSegment.GetMaterialMappingControl().SetFaceColor(new HPS.RGBAColor(1.0f, 0, 0, 0.5f));

            // create the textured style based on the image we loaded earlier
            HPS.SegmentKey texturedStyleKey = HPS.Database.CreateRootSegment();
            HPS.NamedStyleDefinition textured_style = myPortfolio.DefineNamedStyle("textured_style", texturedStyleKey);
            textured_style.GetSource().GetMaterialMappingControl().SetFaceTexture("my_texture");

            // set up the highlight options kits
            HPS.HighlightOptionsKit hok1 = new HPS.HighlightOptionsKit("red_style");
            HPS.HighlightOptionsKit hok2 = new HPS.HighlightOptionsKit("transparent_style");
            HPS.HighlightOptionsKit hok3 = new HPS.HighlightOptionsKit("textured_style");


            // make the portfolio active
            modelKey.GetPortfolioControl().Push(myPortfolio);

            view.FitWorld();

            //Setup a legend for the first subwindow
            SegmentKey one_legend = one.Subsegment();
            CameraKit legend_camera = CameraKit.GetDefault();
            legend_camera.SetProjection(Camera.Projection.Stretched);
            one_legend.SetCamera(legend_camera);
            TextKit textKit = new TextKit();
            textKit.SetText("Unstyled model").SetPosition(new Point(0, 0, -0.8f)).SetAlignment(Text.Alignment.Center);
            one_legend.InsertText(textKit);
            one_legend.GetDrawingAttributeControl().SetDepthRange(0, 0); // bring to front

            //Setup a legend for the second subwindow
            SegmentKey two_legend = two.Subsegment();
            two_legend.SetCamera(legend_camera);
            textKit.SetText("Styled with opaque red style");
            two_legend.InsertText(textKit);
            two_legend.GetDrawingAttributeControl().SetDepthRange(0, 0); // bring to front

            //Obtain a key path to the second subwindow, and highlight it using the red style setup earlier
            SprocketPath sprocket_path = new SprocketPath(canvas, canvas.GetAttachedLayout(), view, model);
            KeyPath sprocket_key_path = sprocket_path.GetKeyPath();
            KeyPath two_path = new KeyPath();
            two_path.Append(two);
            two_path.Append(sprocket_key_path);
            canvas.GetWindowKey().GetHighlightControl().Highlight(two_path, hok1);

            //Setup a legend for the third subwindow
            SegmentKey three_legend = three.Subsegment();
            three_legend.SetCamera(legend_camera);
            textKit.SetText("Styled with transparent red style");
            three_legend.InsertText(textKit);
            three_legend.GetDrawingAttributeControl().SetDepthRange(0, 0); // bring to front

            // overlay "InPlace" is necessary for transparent styles
            hok2.SetOverlay(Drawing.Overlay.InPlace);
            KeyPath three_path = new KeyPath();
            three_path.Append(three);
            three_path.Append(sprocket_key_path);
            canvas.GetWindowKey().GetHighlightControl().Highlight(three_path, hok2);

            //Setup a legend for the fourth subwindow
            SegmentKey four_legend = four.Subsegment();
            four_legend.SetCamera(legend_camera);
            textKit.SetText("Styled with textured style");
            four_legend.InsertText(textKit);
            four_legend.GetDrawingAttributeControl().SetDepthRange(0, 0); // bring to front
            KeyPath four_path = new KeyPath();
            four_path.Append(four);
            four_path.Append(sprocket_key_path);
            canvas.GetWindowKey().GetHighlightControl().Highlight(four_path, hok3);

            canvas.Update();
        }
    }
}
